import io

import numpy as np
from PIL import Image


def byte_to_rgb_ndarr(image_bytes: bytes):
    """
    Convert image from `byte` to `ndarray(rgb)`
    """
    bytes_load = io.BytesIO(image_bytes)
    image_pillow = Image.open(bytes_load)
    image_ndarray = pillow_to_rgb_ndarr(image_pillow)

    return image_ndarray


def pillow_to_rgb_ndarr(image_pillow):
    """
    Convert image from `Pillow` to `ndarray(rgb)`
    """
    image_ndarray = np.array(image_pillow.convert('RGB'))

    return image_ndarray


def cv2_to_rgb_ndarr(image_cv2):
    """
    Convert color order from BGR to RGB

    Requirements: `cv2(opencv-python)`
    """
    import cv2

    image_ndarray = cv2.cvtColor(image_cv2, cv2.COLOR_BGR2RGB)

    return image_ndarray



import numpy as np
from sklearn.cluster import KMeans

from majocol.utils import is_positive_int, rgb2hex
from majocol.types import HexRgbList


def pick(image_ndarray, count: int = 1) -> HexRgbList:
    """
    Pick major colors from image
    """
    if not is_positive_int(count):
        return None

    image_height = image_ndarray.shape[0]
    image_width = image_ndarray.shape[1]

    image_size = image_height * image_width
    rgb_ndarray = image_ndarray.reshape(image_size, 3)

    cluster = KMeans(n_clusters=count)
    cluster.fit(X=rgb_ndarray)
    centers_arr = cluster.cluster_centers_.astype(np.uint8, copy=False)

    hex_colors = [rgb2hex(*dec_colors) for dec_colors in centers_arr]
    return hex_colors



from functools import lru_cache
from typing import Any


def is_positive_int(target: Any) -> bool:
    """
    Whether `target` is positive integer or not
    """
    return isinstance(target, int) and target > 0


@lru_cache(maxsize=None)
def rgb2hex(r, g, b):
    """
    Convert RGB color from decimal to hexadecimal
    """
    return '#%02x%02x%02x' % (r, g, b)
